/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (C) 2011-2012  Linagora
 *
 * This program is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU Affero General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version, provided you comply 
 * with the Additional Terms applicable for OBM connector by Linagora 
 * pursuant to Section 7 of the GNU Affero General Public License, 
 * subsections (b), (c), and (e), pursuant to which you must notably (i) retain 
 * the “Message sent thanks to OBM, Free Communication by Linagora” 
 * signature notice appended to any and all outbound messages 
 * (notably e-mail and meeting requests), (ii) retain all hypertext links between 
 * OBM and obm.org, as well as between Linagora and linagora.com, and (iii) refrain 
 * from infringing Linagora intellectual property rights over its trademarks 
 * and commercial brands. Other Additional Terms apply, 
 * see <http://www.linagora.com/licenses/> for more details. 
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License 
 * for more details. 
 *
 * You should have received a copy of the GNU Affero General Public License 
 * and its applicable Additional Terms for OBM along with this program. If not, 
 * see <http://www.gnu.org/licenses/> for the GNU Affero General Public License version 3 
 * and <http://www.linagora.com/licenses/> for the Additional Terms applicable to 
 * OBM connectors. 
 * 
 * ***** END LICENSE BLOCK ***** */

package org.obm.push.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.nio.charset.Charset;

/**
 * File manipulation functions
 * 
 * @author tom
 * 
 */
public class FileUtils {

	private static final int BUFF_SIZE = 100000;

	/**
	 * File copy, from Java Performance book
	 * 
	 * @param src
	 * @param dest
	 * @throws IOException
	 */
	public static void copy(File src, File dest) throws IOException {
		InputStream in = null;
		OutputStream out = null;
		in = new FileInputStream(src);
		out = new FileOutputStream(dest);
		transfer(in, out, true);
	}

	/**
	 * Fast stream transfer method
	 * 
	 * @param in
	 * @param out
	 * @throws IOException
	 */
	public static void transfer(InputStream in, OutputStream out,
			boolean closeIn) throws IOException {
		final byte[] buffer = new byte[BUFF_SIZE];

		try {
			while (true) {
				int amountRead = in.read(buffer);
				if (amountRead == -1) {
					break;
				}
				out.write(buffer, 0, amountRead);
			}
		} finally {
			if (closeIn) {
				in.close();
			}
			out.flush();
			out.close();
		}
	}

	public static String streamString(InputStream in, boolean closeIn)
			throws IOException {
		return new String(streamBytes(in, closeIn), "utf-8");
	}

	public static String streamString(InputStream in, boolean closeIn,
			String charset) throws IOException {
		return new String(streamBytes(in, closeIn), Charset.forName(charset));
	}

	public static byte[] streamBytes(InputStream in, boolean closeIn)
			throws IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		transfer(in, out, closeIn);
		return out.toByteArray();
	}

	public static InputStream dumpStream(InputStream in, PrintStream dump,
			boolean closeIn) throws IOException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			transfer(in, out, closeIn);
			dump.println("-- stream dump start --");
			dump.println(out.toString());
			dump.println("-- stream dump end --");
		} catch (Throwable t) {
			throw new IOException(t);
		}
		return new ByteArrayInputStream(out.toByteArray());
	}

}
